/*
 *	data.c
 */


#include "scheme.h"
#include "arch.h"
#include "memory.h"
#include "reg.h"
#include "heap.h"
#include "object.h"
#include "data.h"


#define ABS(x)		((x) > 0 ? (x) : (- (x)))
#define	A_SPLIT		((BITS_PER_FIXNUM - 1) / 2)
#define B_SPLIT		(BITS_PER_FIXNUM - A_SPLIT)
#define MAX_C		(1 << (BITS_PER_FIXNUM - 1))


times_carefully(x, y)
int x, y;
{
	int a, b, positive_result;
	int low_a, low_b, low_c, high_a, high_b, middle_c, c;
	int middle_c_1, middle_c_2;

	a = extract_fixnum(x);
	b = extract_fixnum(y);

	positive_result = (a >= 0) ? (b >= 0) : (b < 0);

	a = ABS(a);
	b = ABS(b);

	low_a = a & ((1 << A_SPLIT) - 1);
	low_b = b & ((1 << B_SPLIT) - 1);
	low_c = low_a * low_b;

	if (low_c > MAX_C)
		return ILLEGAL_FIXNUM;

	high_a = a >> A_SPLIT;
	high_b = b >> A_SPLIT;

	if (high_a > 0 && high_b > 0)
	{
		if (! positive_result
		    && (a == (1 << A_SPLIT))
		    && (b == (1 << B_SPLIT)))
		{
			assert(- (a * b) == LEAST_FIXNUM_VALUE);
			return enter_fixnum(LEAST_FIXNUM_VALUE);
		}
		else
			return ILLEGAL_FIXNUM;
	}
	else
	{
		middle_c_1 = high_a * low_b;
		middle_c_2 = high_b * low_a;
		if ((middle_c_1 > (1 << B_SPLIT))
		    || (middle_c_2 > (1 << A_SPLIT)))
			return ILLEGAL_FIXNUM;
		middle_c = (middle_c_1 << A_SPLIT)
			+ (middle_c_2 << B_SPLIT);
		if (middle_c > MAX_C)
			return ILLEGAL_FIXNUM;
		
		c = low_c + middle_c;
		if (c > MAX_C)
			return ILLEGAL_FIXNUM;

		if (positive_result)
			if (c == MAX_C)
				return ILLEGAL_FIXNUM;
			else
				return enter_fixnum(c);
		else
			return enter_fixnum(- c);
	}
}



header_type(h)
int h;
{
	assert(HEADER_P(h));
	return DESCRIPTOR_DATA(h) & ((1 << HEADER_TYPE_FIELD_WIDTH) - 1);
}


header_length_in_bytes(h)
int h;
{
	assert(HEADER_P(h));
	return DESCRIPTOR_DATA(h) >> HEADER_TYPE_FIELD_WIDTH;
}


address_after_header(stob)
int stob;
{
	assert(STOB_P(stob));
	return stob - TAG_STOB;
}


stob_equal_p(stob1, stob2)
int stob1, stob2;
{
	int i, z1, z2, length;

	z1 = STOB_HEADER(stob1);
	z2 = STOB_HEADER(stob2);

	if (z1 != z2)
		return FALSE;

	length = HEADER_LENGTH_IN_CELLS(z1);
	for (i = 0; i < length; i++)
		if (STOB_REF(stob1, i) != STOB_REF(stob2, i))
			return FALSE;
	return TRUE;
}


make_stob(type, length)
{
	int new;

	assert(AVAILABLE_P(BYTES_TO_CELLS(length) + 1));
	memory->data[reg_heap++] = MAKE_HEADER(type, length);

	new = MAKE_STOB_DESCRIPTOR(reg_heap);
	reg_heap += BYTES_TO_AUNITS(length);
	if (length > 0)
		memory->data[reg_heap - 1] = 0;
	return new;
}


/*
 *  Vectors of descriptors.
 */


d_vector_length(x)
int x;
{
	assert(D_VECTOR_P(x));
	return HEADER_LENGTH_IN_CELLS(STOB_HEADER(x));
}


d_vector_ref(x, index)
int x;
int index;
{
	assert(VALID_INDEX_P(index, d_vector_length(x)));
	return STOB_REF(x, index);
}


d_vector_set(x, index, val)
int x;
int index;
int val;
{
	assert(VALID_INDEX_P(index, d_vector_length(x)));
	STOB_SET(x, index, val);
}

/*
 *  Vectors of bytes.
 */



b_vector_length(x)
int x;
{
	assert(B_VECTOR_P(x));
	return header_length_in_bytes(STOB_HEADER(x));
}


b_vector_ref(x, i)
int x, i;
{
	assert(VALID_INDEX_P(i, b_vector_length(x)));
	return ((BYTE *) address_after_header(x))[i];
}


b_vector_set(x, i, val)
int x, i, val;
{
	assert(VALID_INDEX_P(i, b_vector_length(x)));
	((BYTE *) address_after_header(x))[i] = val;
}
