/*
 *	debug.c
 */

#include <stdio.h>
#include "scheme.h"
#include "arch.h"
#include "memory.h"
#include "object.h"
#include "data.h"
#include "heap.h"

void show_range();
void describe(), describe_immediate(), describe_header();


#ifdef notdef

void
show_memory()
{
	show_range(newspace_begin, newspace_end);
}

void
show_all_memory()
{
	show_range(memory->begin, memory->end);
}


void
show_range(start, end)
int start, end;
{
	int i;

	for (i = start; i < end; )
		i = show_location(i);
}


show_location(index)
int index;
{
	int desc;

	desc = memory->data[index];

	fprintf(stdout, "%d:  ", index);
	describe(desc);
	if (DESCRIPTOR_TAG(desc) == TAG_HEADER
	    && (header_type(desc) == STOB_STRING
		|| header_type(desc) == STOB_CODE_VECTOR))
		return describe_byte_vector(index);

	return index + 1;
}


void
describe(desc)
int desc;
{
	if (desc == NULL_OBJ)
	{
		fprintf(stdout, "()\n");
		return;
	}
	
	switch (DESCRIPTOR_TAG(desc))
	{
	    case TAG_FIXNUM:
		fprintf(stdout, "fix  %d", extract_fixnum(desc));
		break;
	    case TAG_STOB:
		fprintf(stdout, "stob %d", address_after_header(desc));
		break;
	    case TAG_HEADER:
		fprintf(stdout, "hdr  ");
		describe_header(desc);
		break;
	    case TAG_IMMEDIATE:
		fprintf(stdout, "imm  ");
		describe_immediate(desc);
		break;
	    default:
		error("Unknown object");
	}
	fprintf(stdout, "\n");
}


void
describe_immediate(imm)
int imm;
{
	switch(immediate_type(imm))
	{
	    case IMM_FALSE:
		fprintf(stdout, "#f");
		break;
	    case IMM_TRUE:
		fprintf(stdout, "#t");
		break;
	    case IMM_CHAR:
		fprintf(stdout, "char '%c' = ", immediate_info(imm));
		break;
	    case IMM_UNSPECIFIED:
		fprintf(stdout, "unspecified");
		break;
	    case IMM_UNDEFINED:
		fprintf(stdout, "undefined");
		break;
	    case IMM_EOF:
		fprintf(stdout, "EOF");
		break;
	    default:
		error("Illegal immediate type -- %d", immediate_type(imm));
	}
	fprintf(stdout, " %d", immediate_info(imm));
}


void
describe_header(hdr)
int hdr;
{
	switch(header_type(hdr))
	{
	    case STOB_PAIR:
		fprintf(stdout, "pair");
		break;
	    case STOB_SYMBOL:
		fprintf(stdout, "symbol");
		break;
	    case STOB_VECTOR:
		fprintf(stdout, "vector");
		break;
	    case STOB_CLOSURE:
		fprintf(stdout, "closure");
		break;
	    case STOB_CELL:
		fprintf(stdout, "cell");
		break;
	    case STOB_PORT:
		fprintf(stdout, "port");
		break;
	    case STOB_RATIO:
		fprintf(stdout, "ratio");
		break;
	    case STOB_STRING:
		fprintf(stdout, "string");
		break;
	    case STOB_CODE_VECTOR:
		fprintf(stdout, "code vector");
		break;
	    case STOB_DOUBLE:
		fprintf(stdout, "double");
		break;
	    case STOB_BIGNUM:
		fprintf(stdout, "bignum");
		break;
	    default:
		error("Illegal stored object type.");
	}
	fprintf(stdout, " %d", header_length_in_bytes(hdr));
}


describe_byte_vector(index)
int index;
{
	int i, value, stob, desc;

	desc = memory->data[index];
	stob = MAKE_STOB_DESCRIPTOR(index + 1);
	fprintf(stdout, "\t");

	for (i = 0; i < header_length_in_bytes(desc); i++)
	{
		value = b_vector_ref(stob, i);
		if (header_type(desc) == STOB_STRING)
			fprintf(stdout, "%c", value);
		else if (header_type(desc) == STOB_CODE_VECTOR)
			describe_opcode(value);
	}
	fprintf(stdout, "\n");
	(void) fflush(stdout);
	return index + 1 + BYTES_TO_CELLS(header_length_in_bytes(desc));
}


describe_opcode(op)
int op;
{
	fprintf(stdout, "(%d)", op);
}



print_info(opcode)
int opcode;
{
	printf("OP %d\n", opcode);
	printf("    pc %d\n", REG_PC);
	printf("    val %d\n", reg_val);
	printf("    heap %d\n", reg_heap);
	printf("    nargs %d\n", reg_nargs);
	if (reg_nargs == 2)
		printf("    arg2 %d\n", reg_arg2);
	if (reg_nargs == 3)
	{
		printf("    arg2 %d\n", reg_arg2);
		printf("    arg3 %d\n", reg_arg3);
	}
	(void) fflush(stdout);
}

#endif
