/*
 *	gc.c -- garbage collection and heap dumping
 */


#include "scheme.h"
#include "arch.h"
#include "reg.h"
#include "object.h"
#include "data.h"			/* for STOB_VECTOR */
#include "heap.h"
#include "memory.h"			/* for memory */
#include "vmio.h"			/* for port stuff */


/*
 *  A vector containing the root and the stack is created in the newspace
 *  after the garbage collection flip to start off the scanning process.
 */
#define ROOT_SIZE		2	/* size of the vector */
#define ROOT_LOCATION		0	/* position of root pointer */
#define STACK_LOCATION		1	/* position of stack pointer */


allow_marginal_allocation()
{
	reg_limit = newspace_end;
}


collect()
{
	int temp;

	fprintf(stderr, "Garbage collecting . . .");
	(void) fflush(stderr);

	/*
	 *  Flip oldspace and newspace.
	 */
	temp = newspace_begin;
	newspace_begin = oldspace_begin;
	oldspace_begin = temp;
	temp = newspace_end;
	newspace_end = oldspace_end;
	oldspace_end = temp;

	reg_limit = newspace_end - MARGIN;
	reg_heap = newspace_begin;

	/*
	 *	Create vector containing root and stack pointers in newspace.
	 */
	temp = MAKE_D_VECTOR(STOB_VECTOR, ROOT_SIZE);
	d_vector_set(temp, ROOT_LOCATION, reg_root);
	d_vector_set(temp, STACK_LOCATION, reg_stack);

	/*
	 *  Collect the garbage.
	 */
	scan();

	fprintf(stderr, " done.\n");
	(void) fflush(stderr);
}


scan()
{
	int i, p, r, thing, h, new, new_heap;
	CELL *q;

	for (p = newspace_begin; p < reg_heap && reg_heap < reg_limit; p++)
	{
		thing = memory->data[p];
		if (B_VECTOR_HEADER_P(thing))
			p += HEADER_AUNITS(thing);
		else if (in_oldspace(thing))
		{
			h = STOB_HEADER(thing);
			if (STOB_P(h))	/* broken heart */
				memory->data[p] = h;
			else
			{
				memory->data[reg_heap++] = h;

				new = MAKE_STOB_DESCRIPTOR(reg_heap);
				STOB_HEADER_SET(thing, new); /* break heart */
				memory->data[p] = new;
				
				new_heap = reg_heap + HEADER_AUNITS(h);
				for (q = (CELL *) address_after_header(thing);
				     reg_heap < new_heap;
				     q++)
				{
					r = *q;
					assert(B_VECTOR_HEADER_P(h)
					       || ! STOB_P(r)
					       || in_oldspace(r));
					memory->data[reg_heap++] = r;
				}
			}
		}
	}

	if (reg_heap >= reg_limit)
		error("Out of memory in garbage collection.");

	/*
	 *  Adding one is necessary here to skip over the object header.
	 */
	reg_root = memory->data[newspace_begin + 1 + ROOT_LOCATION];
	reg_stack = memory->data[newspace_begin + 1 + STACK_LOCATION];

	for (i = 2; i < MAX_PORTS; i++)	/* XXX we skip input and output */
		if (port_table[i].fp)
			if (! STOB_HEADER(port_table[i].port))
				close_port_noisily(port_table[i].port);

}


in_oldspace(desc)
int desc;
{
	CELL *addr;

	if (STOB_P(desc))
	{
		addr = (CELL *) address_after_header(desc);
		return (addr >= &memory->data[oldspace_begin])
			&& (addr < &memory->data[oldspace_end]);
	}
	return FALSE;
}
