/*
 *	object.h
 */


#ifndef _OBJECT_H
#define _OBJECT_H

/*
 *  The heap contains objects (descriptors) which are tagged.
 */

#define TAG_FIELD_WIDTH			2
#define DATA_DIELD_WIDTH		(BITS_PER_CELL - TAG_FIELD_WIDTH)

#define MAKE_DESCRIPTOR(tag, data)	(((data) << TAG_FIELD_WIDTH) | (tag))
#define DESCRIPTOR_TAG(desc)		((desc) & ((1 << TAG_FIELD_WIDTH) - 1))
#define DESCRIPTOR_DATA(desc)		((desc) >> TAG_FIELD_WIDTH)


/*
 *  Tags on the data in the heap.
 */

#define TAG_FIXNUM	0
#define TAG_IMMEDIATE	1
#define TAG_HEADER	2
#define TAG_STOB	3

/*
 *  Predicates on those tags.
 */

#define FIXNUM_P(desc)		(DESCRIPTOR_TAG(desc) == TAG_FIXNUM)
#define IMMEDIATE_P(desc)	(DESCRIPTOR_TAG(desc) == TAG_IMMEDIATE)
#define HEADER_P(desc)		(DESCRIPTOR_TAG(desc) == TAG_HEADER)
#define STOB_P(desc)		(DESCRIPTOR_TAG(desc) == TAG_STOB)
#define NUMBER_P(desc)		FIXNUM_P(desc)

/* if these are not used anywhere, flush them!!! XXX */
#define COMPLEX_P(desc)		FIXNUM_P(desc)
#define REAL_P(desc)		FIXNUM_P(desc)
#define RATIONAL_P(desc)	FIXNUM_P(desc)
#define INTEGER_P(desc)		FIXNUM_P(desc)

/*
 *  Fixed length numbers.
 */

#define BITS_PER_FIXNUM	(BITS_PER_CELL - TAG_FIELD_WIDTH)
#define LEAST_FIXNUM_VALUE	(- (1 << BITS_PER_FIXNUM))
#define GREATEST_FIXNUM_VALUE	((1 << BITS_PER_FIXNUM) - 1)

#define TOO_BIG_FOR_FIXNUM_P(n)		((n) > GREATEST_FIXNUM_VALUE)
#define TOO_SMALL_FOR_FIXNUM_P(n)	((n) < LEAST_FIXNUM_VALUE)
#define OVERFLOWS_P(n)			(TOO_BIG_FOR_FIXNUM_P(n)	\
					 || TOO_SMALL_FOR_FIXNUM_P(n))
#define ILLEGAL_FIXNUM		(GREATEST_FIXNUM_VALUE + 1)


#define ENTER_BOOLEAN(b)	((b) ? TRUE_OBJ : FALSE_OBJ)
#define ENTER_CHAR(c)		MAKE_IMMEDIATE(IMM_CHAR, (c))



/*
 *	Immediate types.
 */

#define IMM_FALSE		0
#define IMM_TRUE		1
#define IMM_CHAR		2
#define IMM_UNSPECIFIED		3
#define IMM_UNDEFINED		4
#define IMM_EOF			5

#define IMMEDIATE_TYPE_FIELD_WIDTH	(8 - TAG_FIELD_WIDTH)

#define MAKE_IMMEDIATE(type, info)					\
		MAKE_DESCRIPTOR(TAG_IMMEDIATE,				\
				(((info) << IMMEDIATE_TYPE_FIELD_WIDTH)	\
				 | (type)))
							


#define FALSE_P(d)	(IMMEDIATE_P(d) && immediate_type(d) == IMM_FALSE)
#define CHAR_P(d)	(IMMEDIATE_P(d) && immediate_type(d) == IMM_CHAR)
#define UNDEFINED_P(d)	(IMMEDIATE_P(d) && immediate_type(d) == IMM_UNDEFINED)

#define TRUE_OBJ	MAKE_IMMEDIATE(IMM_TRUE, 0)
#define FALSE_OBJ	MAKE_IMMEDIATE(IMM_FALSE, 0)
#define NULL_OBJ	MAKE_IMMEDIATE(IMM_FALSE, 1)
#define EOF_OBJ		MAKE_IMMEDIATE(IMM_EOF, 0)
#define UNSPECIFIED_OBJ	MAKE_IMMEDIATE(IMM_UNSPECIFIED, 0)

#define QUIESCENT_OBJ		MAKE_IMMEDIATE(IMM_UNDEFINED, 0)
#define UNBOUND_MARKER_OBJ	MAKE_IMMEDIATE(IMM_UNDEFINED, 1)
#define UNASSIGNED_MARKER_OBJ	MAKE_IMMEDIATE(IMM_UNDEFINED, 2)

#endif
