/*
 *	op.h
 */


#ifndef _OP_H_
#define _OP_H_

/*
 *  Register manipulation.
 */

#define OP_CHECK_NARGS_EQ		0
#define OP_CHECK_NARGS_GE		1
#define OP_MAKE_ENV			2
#define OP_MAKE_REST_LIST		3
#define OP_LITERAL			4
#define OP_LOCAL			5
#define OP_SET_LOCAL			6
#define OP_GLOBAL			7
#define OP_SET_GLOBAL			8
#define OP_CLOSURE			9
#define OP_PUSH				10
#define OP_POP				11
#define OP_MAKE_CONT			12
#define OP_CALL				13
#define OP_JUMP_IF_FALSE		14
#define OP_JUMP				15
#define OP_RETURN			16
#define OP_PUSH_CONT			17
#define OP_POP_CONT			18
#define OP_SPREAD_ARGS			19
#define OP_N_CALL			20
#define OP_NATIVE			21

/*
 *  Scalar primitives.
 */

#define OP_EQ_P				22
#define OP_INTEGER_P			23
#define OP_NUMBER_P			24
#define OP_PLUS				25
#define OP_MINUS			26
#define OP_TIMES			27
#define OP_EQUAL			28
#define OP_LESS_THAN			29
#define OP_QUOTIENT			30
#define OP_REMAINDER			31
#define OP_CHAR_P			32
#define OP_CHAR_EQUAL_P			33
#define OP_CHAR_LESS_P			34
#define OP_CHAR_TO_ASCII		35
#define OP_ASCII_TO_CHAR		36
#define OP_EOF_OBJECT_P			37

/*
 *  Data manipulation.
 */

#define OP_PAIR_P			38
#define OP_CONS				39
#define OP_CAR				40
#define OP_CDR				41
#define OP_SET_CAR			42
#define OP_SET_CDR			43
#define OP_SYMBOL_P			44
#define OP_MAKE_SYMBOL			45
#define OP_SYMBOL_TO_STRING		46
#define OP_CELL_P			47
#define OP_MAKE_CELL			48
#define OP_CELL_NAME			49
#define OP_CONTENTS			50
#define OP_SET_CONTENTS			51
#define OP_CLOSURE_P			52
#define OP_MAKE_CLOSURE			53
#define OP_CLOSURE_ENV			54
#define OP_CLOSURE_TEMPLATE		55
#define OP_CODE_VECTOR_P		56
#define OP_MAKE_CODE_VECTOR		57
#define OP_CODE_VECTOR_LENGTH		58
#define OP_CODE_VECTOR_REF		59
#define OP_CODE_VECTOR_SET		60
#define OP_STRING_P			61
#define OP_MAKE_STRING			62
#define OP_STRING_LENGTH		63
#define OP_STRING_REF			64
#define OP_STRING_SET			65
#define OP_VECTOR_P			66
#define OP_MAKE_VECTOR			67
#define OP_VECTOR_LENGTH		68
#define OP_VECTOR_REF			69
#define OP_VECTOR_SET			70

/*
 *  Input and output primitives.
 */

#define OP_OPEN_PORT			71
#define OP_CLOSE_PORT			72
#define OP_INPUT_PORT_P			73
#define OP_OUTPUT_PORT_P		74
#define OP_READ_CHAR			75
#define OP_PEEK_CHAR			76
#define OP_WRITE_CHAR			77
#define OP_WRITE_STRING			78

/*
 *  Miscellaneous stuff.
 */

#define OP_UNASSIGNED			79
#define OP_HALT				80
#define OP_SET_ENABLED_INTERRUPTS	81
#define OP_RETURN_FROM_HANDLER		82
#define OP_WRITE_IMAGE			83

/*
 *  Unnecessary primitives
 */

#define OP_STRING_EQUAL_P		84
#define OP_STRING_HASH			85
#define OP_REVERSE_LIST_TO_STRING	86
#define OP_INTERN			87
#define OP_LOOKUP			88

/*
 *  New stuff
 */

#define OP_RATIONAL_P			89
#define OP_REAL_P			90
#define OP_COMPLEX_P			91
#define OP_DIVIDE			92
#define OP_COLLECT			93
#define OP_GET_DYNAMIC_STATE		94
#define OP_SET_DYNAMIC_STATE		95

#endif
