/*
 *	struct.c
 */


#include "scheme.h"
#include "arch.h"
#include "data.h"
#include "struct.h"
#include "object.h"


/*
 *  This file contains routines for manipulating structures in the virtual
 *  machine heap.
 */


cons(car, cdr)
int car, cdr;
{
	int ans = MAKE_D_VECTOR(STOB_PAIR, PAIR_LENGTH);

	d_vector_set(ans, 0, car);
	d_vector_set(ans, 1, cdr);
	return ans;
}


make_symbol(x)
int x;
{
	int ans = MAKE_D_VECTOR(STOB_SYMBOL, SYMBOL_LENGTH);

	d_vector_set(ans, 0, x);
	return ans;
}


make_closure(template, env)
int template, env;
{
	int ans = MAKE_D_VECTOR(STOB_CLOSURE, CLOSURE_LENGTH);

	d_vector_set(ans, 0, template);
	d_vector_set(ans, 1, env);
	return ans;
}


make_cell(contents, name)
int contents, name;
{
	int ans = MAKE_D_VECTOR(STOB_CELL, CELL_LENGTH);

	d_vector_set(ans, 0, contents);
	d_vector_set(ans, 1, name);
	return ans;
}


make_port(mode, index, peek, id)
int mode, index, peek, id;
{
	int ans = MAKE_D_VECTOR(STOB_PORT, PORT_LENGTH);

	d_vector_set(ans, 0, mode);
	d_vector_set(ans, 1, index);
	d_vector_set(ans, 2, peek);
	d_vector_set(ans, 3, id);
	return ans;
}


char *
extract_string(str)
int str;
{
	int i, len;
	char *v;
	extern char *malloc();

	len = STRING_LENGTH(str);
	if ((v = malloc((unsigned) len + 1)) == (char *) NULL)
		error("no room for string");
	for (i = 0; i < len; i++)
		v[i] = STRING_REF(str, i);
	v[len] = '\0';			/* XXX -- check if needed */
	return v;
}


string_hash(str)
int str;
{
	int i, length, hash;;

	length = STRING_LENGTH(str);
	hash = 0;
	for (i = 0; i < length; i++)
		hash += STRING_REF(str, i);
	return hash;
}


intern(object, table)
int object, table;
{
	int index, bucket, b, new;

	index = string_hash(object) & (VECTOR_LENGTH(table) - 1);
	bucket = VECTOR_REF(table, index);
	b = bucket;
	while (1)
	{
		if (b == NULL_OBJ)
		{
			new = make_symbol(object);
			VECTOR_SET(table, index, cons(new, bucket));
			return new;
		}
		if (STRING_EQUAL_P(object, SYMBOL_TO_STRING(CAR(b))))
			return CAR(b);
		b = CDR(b);
	}
}


lookup(object, table)
int object, table;
{
	int index, bucket, b, new;

	index = string_hash(SYMBOL_TO_STRING(object))
		& (VECTOR_LENGTH(table) - 1);
	bucket = VECTOR_REF(table, index);
	b = bucket;
	while (1)
	{
		if (b == NULL_OBJ)
		{
			new = make_cell(UNBOUND_MARKER_OBJ, object);
			VECTOR_SET(table, index, cons(new, bucket));
			return new;
		}
		if (object == CELL_NAME(CAR(b)))
			return CAR(b);
		b = CDR(b);
	}
}
