/*
 *	vmio.c -- input and output primitives
 */


#include <setjmp.h>
#include "scheme.h"
#include "arch.h"
#include "ex.h"
#include "object.h"
#include "struct.h"
#include "vmio.h"
#include "reg.h"


FILE *extract_port();

extern void handle_interrupt();
extern void raise_exception();


jmp_buf read_char_context;
int waiting_in_read;


void
initialize_io_system()
{
	int i;

	for (i = 0; i < MAX_PORTS; i++)
		port_table[i].fp = (FILE *) NULL;
	port_table[0].fp = stdin;
	port_table[1].fp = stdout;
}


OPEN_P(port)
int port;
{
	return extract_fixnum(PORT_INDEX(port)) >= 0;
}


close_port_noisily(port)
int port;
{
	close_port(port);

	/* XXX gc problems here?? -- could port go away?? */
	printf("Port closed:  %s\n", extract_string(PORT_ID(port)));
}


open_port(filename, mode)
int filename;
int mode;
{
	int index, port;
	char *filestring;
	FILE *fp;

	for (index = 0; index < MAX_PORTS; index++)
		if (! port_table[index].fp)
			break;

	if (index == MAX_PORTS)
		/* should do collect and retry here! XXX */
		error("Couldn't open file.");

	filestring = extract_string(filename);
	if (mode == FOR_INPUT)
		fp = fopen(filestring, "r");
	else				/* for output */
		fp = fopen(filestring, "w");

	free(filestring);

	if (fp == (FILE *) NULL)
		raise_exception(EX_CANNOT_OPEN);

	port = make_port(enter_fixnum(mode),
			 enter_fixnum(index),
			 FALSE_OBJ,
			 filename);

	port_table[index].port = port;
	port_table[index].fp = fp;

	return port;
}


close_port(port)
int port;
{
	int index;
	FILE *fp;

	if (! OPEN_P(port))
		return;

	index = extract_fixnum(PORT_INDEX(port));

	/*
	 *  Do not allow Scheme code to close standard input, or output.
	 */
	if (index == 0 || index == 1)
		return;

	fp = extract_port(port);
	(void) fclose(fp);
	SET_PORT_MODE(port, 0);		/* shouldn't we enter_fixnum here? */
	SET_PORT_INDEX(port, -1);	/* and here too XXX */
	port_table[index].fp = (FILE *) NULL;
	port_table[index].port = -1;
}


read_char(port)
int port;
{
	int ch;

	if (! OPEN_P(port))
		raise_exception(EX_OPERATION_ON_CLOSED_PORT);

	if ((ch = PORT_PEEKED_CHAR(port)) != FALSE_OBJ)
	{
		SET_PORT_PEEK(port, FALSE_OBJ);
		return ch;
	}

	return raw_read_char(port);
}


raw_read_char(port)
int port;
{
	int ch;

    restart_read:

	if (! setjmp(read_char_context))
	{
		waiting_in_read = TRUE;
		if (reg_pending_interrupt != INT_NONE)
			return ENTER_CHAR_OR_EOF(EOF);
		ch = ENTER_CHAR_OR_EOF(getc(extract_port(port)));
		waiting_in_read = FALSE;
		return ch;
	}
	else
	{
		/*
		 *  Our read was interrupted by a signal.
		 */
		handle_interrupt();
		goto restart_read;
	}
}


peek_char(port)
int port;
{
	int ch;

	if (! OPEN_P(port))
		raise_exception(EX_OPERATION_ON_CLOSED_PORT);

	if ((ch = PORT_PEEKED_CHAR(port)) == FALSE_OBJ)
	{
		ch = raw_read_char(port);
		SET_PORT_PEEK(port, ch);
	}
	return ch;
}


FILE *
extract_port(port)
int port;
{
	int index;

	index = extract_fixnum(PORT_INDEX(port));
	if (index >= 0)
		return port_table[index].fp;
	error("extract port error");
	/* NOTREACHED */
}


ENTER_CHAR_OR_EOF(ch)
int ch;
{
	if (ch == EOF)
		return EOF_OBJ;
	else
		return ENTER_CHAR(ch);
}
